@@ -1,5 +1,5 @@
 """Functions to support the Extended Python Debugger.
-$Id: fns.py,v 1.42 2007/04/11 09:28:50 rockyb Exp $"""
+$Id: fns.py,v 1.45 2007/05/23 09:25:43 rockyb Exp $"""
 # -*- coding: utf-8 -*-
 #   Copyright (C) 2007 Rocky Bernstein
 #
@@ -36,24 +36,6 @@ def arg_split(s,posix=False):
     lex.whitespace_split = True
     return list(lex)
 
-def count_frames(frame, count_start=0):
-    "Return a count of number of frames"
-    count = -count_start
-    while frame: 
-        count += 1
-        frame = frame.f_back
-    return count
-
-def decorate_fn_with_doc(new_fn, old_fn, additional_text=""):
-    """Make new_fn have old_fn's doc string. This is particularly useful
-    for the do_... commands that hook into the help system.
-    Adapted from from a comp.lang.python posting
-    by Duncan Booth."""
-    def wrapper(*args, **kw):
-        return new_fn(*args, **kw)
-    wrapper.__doc__ = old_fn.__doc__ + additional_text
-    return wrapper
-
 def checkline(obj, filename, lineno):
     """Check whether specified line seems to be executable.
 
@@ -72,6 +54,98 @@ def checkline(obj, filename, lineno):
         return 0
     return lineno
 
+def columnize_array(list, max_elts=50, displaywidth=80):
+    """Display an array as a compact column-aligned set of columns.
+
+    Columns are separated by two spaces (one was not legible enough).
+    """
+    if not list:
+        return "<empty>\n"
+
+    if len(list) > max_elts:
+        list = list[0:max_elts-1]
+        elipsis = True
+    else:
+        elipsis = False
+    nonscalars = [i for i in range(len(list))
+                  if not (type(list[i]) in [types.BooleanType, types.FloatType, 
+                                            types.IntType,  types.StringType,
+                                            types.UnicodeType, types.NoneType,
+                                            types.LongType])]
+    if nonscalars:
+        return ("list[i] not a scalar for i in %s" %
+                ", ".join(map(str, nonscalars)))
+    size = len(list)
+    if size == 1:
+        return '[%s]' % str(list[0])
+    # Consider arranging list in 1 rows total, then 2 rows...
+    # Stop when at the smallest number of rows which
+    # can be arranged less than the display width.
+    for nrows in range(1, len(list)):
+        ncols = (size+nrows-1) // nrows  # ceil(size/nrows)
+        colwidths = []
+        totwidth = -2
+        # get max column width for this column
+        for col in range(ncols):
+            colwidth = 0
+            for row in range(nrows):
+                i = row*ncols + col # [rows, cols]
+                if i >= size:
+                    break
+                x = list[i]
+                colwidth = max(colwidth, len(repr(x)))
+            colwidths.append(colwidth)
+            totwidth += colwidth + 2
+            if totwidth > displaywidth:
+                break
+        if totwidth <= displaywidth:
+            break
+    else:
+        nrows = len(list)
+        ncols = 1
+        colwidths = [0]
+    # The smallest number of rows computed and the
+    # max widths for each column has been obtained.
+    # Now we just have to format each of the
+    # rows.
+    s = '['
+    for row in range(nrows):
+        texts = []
+        for col in range(ncols):
+            i = row*ncols + col
+            if i >= size:
+                x = ""
+            else:
+                x = list[i]
+            texts.append(x)
+        while texts and not texts[-1]:
+            del texts[-1]
+        for col in range(len(texts)):
+            texts[col] = repr(texts[col]).ljust(colwidths[col])
+        s += ("%s\n "%str(", ".join(texts)))
+    s = s[0:-2] 
+    if elipsis: s += "..."
+    s += ']'
+    return s
+
+def count_frames(frame, count_start=0):
+    "Return a count of number of frames"
+    count = -count_start
+    while frame: 
+        count += 1
+        frame = frame.f_back
+    return count
+
+def decorate_fn_with_doc(new_fn, old_fn, additional_text=""):
+    """Make new_fn have old_fn's doc string. This is particularly useful
+    for the do_... commands that hook into the help system.
+    Adapted from from a comp.lang.python posting
+    by Duncan Booth."""
+    def wrapper(*args, **kw):
+        return new_fn(*args, **kw)
+    wrapper.__doc__ = old_fn.__doc__ + additional_text
+    return wrapper
+
 def file_pyc2py(filename):
     """Given a file name, if the suffix is pyo or pyc (an optimized bytecode
     file), change that to the py equivalent"""
@@ -405,3 +479,23 @@ if __name__ == '__main__':
     assert printf(33, "/c") == '!'
     assert printf(33, "/x") == '0x21'
     assert file2module("/tmp/gcd.py") == 'gcd'
+    assert columnize_array(["a"]) == "a\n"
+    print columnize_array([
+            "one", "two", "three",
+            "4ne", "5wo", "6hree",
+            "7ne", "8wo", "9hree",
+            "10e", "11o", "12ree",
+            "13e", "14o", "15ree",
+            "16e", "17o", "18ree",
+            "19e", "20o", "21ree",
+            "22e", "23o", "24ree",
+            "25e", "26o", "27ree",
+            "28e", "29o", "30ree",
+            "31e", "32o", "33ree",
+            "34e", "35o", "36ree",
+            "37e", "38o", "39ree",
+            "40e", "41o", "42ree",
+            "43e", "44o", "45ree",
+            "46e", "47o", "48ree",
+            "one", "two", "three"])
+
