@@ -1,8 +1,8 @@
 """Handles gdb-like command processing.
 (See also pydb.doc for documentation.)
-$Id: gdb.py.in,v 1.131 2007/04/07 10:53:28 rockyb Exp $"""
+$Id: gdb.py.in,v 1.151 2008/05/21 11:02:51 rockyb Exp $"""
 # -*- coding: utf-8 -*-
-#   Copyright (C) 2007 Rocky Bernstein
+#   Copyright (C) 2007, 2008 Rocky Bernstein (rocky@gnu.org)
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@ $Id: gdb.py.in,v 1.131 2007/04/07 10:53:28 rockyb Exp $"""
 _debugger_name = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.22'
+_version = '1.23'
 
 import atexit, inspect, linecache, os, pprint, pydoc, shlex
 import bdb, disassemble, re, subcmd, sys, types
@@ -84,11 +84,13 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
 
         self.all_completions = lambda arg, left=True: \
                                all_completions(self, arg, left)
+        self.autoeval        = True
         self.basename        = False
         self.connection      = None   # For out-of-process communication
         self.currentbp       = None   # for "info program"
         self.cur_frame       = None   # current stack frame
         self.dbg_pydb        = False  # Debug the debugger?
+        self.def_trace       = False  # Show/stop at method create statements?
         self.debug_signal    = None   # The signal used by 'attach'
         self.display         = Display()
         self.fntrace         = False  # Tracing functions/methods
@@ -131,6 +133,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
 
         self.target          = 'local' # local connections by default
         self.target_addr     = ''      # target address used by 'attach'
+        self.width           = 80      # Assume a printed line is this wide
 
         # self.break_anywhere, self.set_continue and
         # self.trace_dispatch are changed depending on the value of
@@ -180,7 +183,12 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
             self.hist_last = 0
             self.hist_save = False
             self.set_history_length = readline.set_history_length
-            self.set_history_length(256)
+            # Set history length using gdb's rule.
+            try:
+                history_length = int(os.environ['HISTSIZE'])
+            except:
+                history_length = 256
+            self.set_history_length(history_length)
 
             # An application like ipython may have its own
             # history set up beforehand, clear that so we don't back up
@@ -335,9 +343,12 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         to work."""
 
         self.setcmds=subcmd.Subcmd("set",   self.help_set.__doc__)
-        self.setcmds.add('args',            self.set_args)
+        self.setcmds.add('annotate',        self.set_annotate, 2)
+        self.setcmds.add('args',            self.set_args, 2)
+        self.setcmds.add('autoeval',        self.set_autoeval, 2)
         self.setcmds.add('basename',        self.set_basename)
-        self.setcmds.add('debug-pydb',      self.set_dbg_pydb)
+        self.setcmds.add('debug-pydb',      self.set_dbg_pydb, 2)
+        self.setcmds.add('deftrace',        self.set_deftrace, 2)
         self.setcmds.add('flush',           self.set_flush)
         self.setcmds.add('fntrace',         self.set_fntrace, 2)
         ## self.setcmds.add('debug-signal',    self.set_debug_signal)
@@ -351,7 +362,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.setcmds.add('sigcheck',        self.set_sigcheck)
         ## self.setcmds.add('target-address',  self.set_target_address)
         self.setcmds.add('trace-commands',  self.set_cmdtrace)
-        self.setcmds.add('warnoptions',     self.set_warnoptions)
+        self.setcmds.add('warnoptions',     self.set_warnoptions, 2)
+        self.setcmds.add('width',           self.set_width,  2)
         return
 
     def __init_show(self):
@@ -360,9 +372,12 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         to work."""
         self.showcmds=subcmd.Subcmd("show", self.help_show.__doc__)
         self.showcmds.add('args',           self.show_args)
+        self.showcmds.add('annotate',       self.show_annotate, 2)
+        self.showcmds.add('autoeval',       self.show_autoeval, 2)
         self.showcmds.add('basename',       self.show_basename)
         self.showcmds.add('debug-pydb',     self.show_dbg_pydb)
         ## self.showcmds.add('debug-signal',   self.show_debug_signal)
+        self.showcmds.add('deftrace',       self.show_deftrace,    3)
         self.showcmds.add('commands',       self.show_commands,    2, False)
         self.showcmds.add('directories',    self.show_directories, 1)
         self.showcmds.add('fntrace',        self.show_fntrace,     2)
@@ -378,7 +393,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         ## self.showcmds.add('target-address', self.show_target_address)
         self.showcmds.add('trace-commands', self.show_cmdtrace,    1)
         self.showcmds.add('version',        self.show_version)
-        self.showcmds.add('warnoptions',    self.show_warnoptions)
+        self.showcmds.add('warnoptions',    self.show_warnoptions, 2)
+        self.showcmds.add('width',          self.show_width, 2)
 
     # To be overridden in derived debuggers
     def defaultFile(self):
@@ -590,13 +606,42 @@ the previous command number shown. A negative number starts from the end."""
         """Generic command for showing command history parameters"""
         try:
             import readline
-            self.msg(
-"""filename:  The filename in which to record the command history is
-"%s".""" % self.histfile)
-            self.msg("save:  Saving of the history record on exit is %s"
-                     % show_onoff(self.hist_save))
-            self.msg("size:  The size of the command history is %d"
-                     % readline.get_history_length())
+            if len(args) > 1 and args[1]:
+                show_save = show_size = show_filename = False
+                show_prefix = False
+                if args[1] == 'filename':
+                    show_filename = True
+                elif args[1] == 'save':
+                    show_save = True
+                elif args[1] == 'size':
+                    show_size = True
+                else:
+                    self.undefined_cmd("show history", args[1])
+                    return
+            else:
+                show_save = show_size = show_filename = True
+                show_prefix = True
+            if show_filename:
+                if show_prefix:
+                    prefix='filename: '
+                else:
+                    prefix=''
+                self.msg("""%sThe filename in which to record the command history is
+"%s".""" % (prefix, self.histfile))
+            if show_save:
+                if show_prefix:
+                    prefix='save: '
+                else:
+                    prefix=''
+                self.msg("%sSaving of the history record on exit is %s"
+                         % (prefix, show_onoff(self.hist_save)))
+            if show_size:
+                if show_prefix:
+                    prefix='size: '
+                else:
+                    prefix=''
+                self.msg("%sThe size of the command history is %d"
+                         % (prefix, readline.get_history_length()))
         except ImportError:
             pass
 
@@ -1301,7 +1346,65 @@ recognized actions include "stop", "nostop", "print", "noprint",
             else:
                 self.msg('Will stop next time breakpoint %d is reached.' %
                          bpnum)
+            return False
+        return False
 
+    def do_ipython(self, arg_str):
+        """ipython [ipython-opt1 ipython-opt2 ...]
+
+Run ipython as a command subshell. You need to have ipython installed
+for this command to work. If no ipython options are
+given the following options are passed: 
+  -noconfirm_exit -prompt_in1 'Pydb In [\#]: '
+"""
+
+        if not self.is_running():
+            return False
+
+        if arg_str:
+            argv = arg_split(arg_str, posix=True)
+        else:
+            argv = ['-noconfirm_exit','-prompt_in1', 'Pydb In [\\#]: ']
+
+        if self.curframe and self.curframe.f_locals:
+            user_ns = self.curframe.f_locals
+        else:
+            user_ns = None
+
+        # IPython does it's own history thing.
+        # Make sure it doesn't damage ours.
+        if self.readline:
+            try:
+                self.write_history_file
+            except IOError:
+                pass
+
+        # ipython should be around - it was before.
+        try:
+            import IPython
+        except ImportError:
+            self.msg("IPython doesn't seem to be importable.")
+            return False
+
+        if user_ns:
+            ipshell = IPython.Shell.IPShellEmbed(argv=argv, user_ns=user_ns)
+        else:
+            ipshell = IPython.Shell.IPShellEmbed(argv=argv)
+
+        if hasattr(ipshell.IP, "magic_pydb"):
+            # We an infinite loop when doing recursive edits
+            self.msg("Removing magic %pydb")
+            delattr(ipshell.IP, "magic_pydb")
+        ipshell()
+        # Restore our history if we can do so.
+        if self.readline and self.histfile is not None:
+            try:
+                self.readline.read_history_file(self.histfile)
+            except IOError:
+                pass
+            return False
+        return False
+                
     def do_info(self, arg):
 
         """Generic command for showing things about the program being
@@ -1326,8 +1429,7 @@ recognized actions include "stop", "nostop", "print", "noprint",
 
         Set the next line that will be executed."""
 
-        if not self.is_running():
-            return False
+        if not self.is_running(): return False
 
         if self.curindex + 1 != len(self.stack):
             self.errmsg("You can only jump within the bottom frame")
@@ -1361,6 +1463,7 @@ we are in interactive mode, we'll prompt to make sure.
                get_confirmation(self, 'Really do a hard kill (y or n)? ',
                                 True):
             os.kill(os.getpid(), signal.SIGKILL)
+            return False # Not reached
 
     def do_list(self, arg):
         """l(ist) [- | first [last or count]]
@@ -1397,6 +1500,8 @@ we are in interactive mode, we'll prompt to make sure.
                     self.lineno = lineno
         except KeyboardInterrupt:
             pass
+        return False
+
     do_l = do_list
 
     def do_next(self, arg):
@@ -1410,7 +1515,7 @@ With an integer argument, perform that many times."""
             self.step_ignore = self.get_pos_int(arg, default=1,
                                                 cmdname='next') - 1
         except ValueError:
-            return
+            return False
 
         self.stepping = True  # Used in thread debugging
         self.set_next(self.curframe)
@@ -1431,8 +1536,24 @@ With an integer argument, perform that many times."""
         """pp expression
         Pretty-print the value of the expression."""
         try:
-            self.msg(pprint.pformat(self.getval(arg)))
-        except:
+            val = self.getval(arg)
+            if type(val) == types.ListType:
+                # Handle simple case where list is not nested
+                simple = True
+                for i in range(len(val)):
+                    if not (type(val[i]) in [types.BooleanType, types.FloatType, 
+                                         types.IntType,  types.StringType,
+                                         types.UnicodeType, types.NoneType,
+                                         types.LongType]):
+                        simple = False
+                        break
+
+                if simple: 
+                    self.msg(fns.columnize_array(val, self.width))
+                    return False
+                
+            self.msg(pprint.pformat(val))
+        except EOFError:
             pass
 
     def do_pwd(self, arg):
@@ -1457,6 +1578,46 @@ source file to document. If name is 'keywords', 'topics', or
         sys.path      = sys_path_save
         return False
 
+    def do_python(self, arg_str):
+        """python 
+
+Run python as a command subshell.
+"""
+
+        # See if python's code module is around
+        try:
+            from code import interact
+        except ImportError:
+            self.msg("python code doesn't seem to be importable.")
+            return False
+
+        # Python does it's own history thing.
+        # Make sure it doesn't damage ours.
+        if self.readline:
+            try:
+                self.write_history_file
+            except IOError:
+                pass
+
+        if self.curframe and self.curframe.f_locals:
+            local = self.curframe.f_locals
+        else:
+            local = None
+
+        if not local:
+            interact(banner='Pydb python shell', local=local)
+        else:
+            interact(banner='Pydb python shell')
+
+        # Restore our history if we can do so.
+        if self.readline and self.histfile is not None:
+            try:
+                self.readline.read_history_file(self.histfile)
+            except IOError:
+                pass
+            return False
+        return False
+
     def do_quit(self, arg):
         """q(uit) or exit - Quit the debugger.  The program being
         executed is aborted."""
@@ -1567,14 +1728,17 @@ assignment to the variable __return__."""
     do_rv = do_retval
 
     def do_run(self, arg_str):
-        """run [arg_str]
+        """run [args...]
 
-        Run/Restart the debugged Python program. If a string (arg_str) is
-        supplied that becomes the new command arguments.  History,
-        breakpoints, actions and debugger options are preserved."""
+        Run or "soft" restart the debugged Python program. If a string
+is supplied, it is splitted with "shlex" but preserving embedded quotes.
+The result is used as the new sys.argv.  History, breakpoints, actions
+and debugger options are preserved. R is a alias for 'run'.
+
+See also 'restart' for an exec-like restart."""
         if not self._program_sys_argv:
-            self.msg("No Python program registered.")
-            self.msg("Perhaps you want to use the 'file' command?")
+            self.errmsg("No Python program registered.")
+            self.errmsg("Perhaps you want to use the 'file' command?")
             return
         if arg_str:
             argv_start = self._program_sys_argv[0:1]
@@ -1585,6 +1749,58 @@ assignment to the variable __return__."""
 
     do_R = do_run
 
+    def do_save(self, arg):
+        """save [all|break|settings] [filename]
+        Save specified settings to a file as a script
+Use the 'source' command in another debug session to restore them."""
+        args = arg.split()
+        actions = ['all', 'break', 'settings']
+        filename = os.path.expanduser("~/pydb-restart.txt")
+        if 0 == len(args):
+            what = 'all'
+        elif 1 == len(args):
+            if args[0] in actions:
+                what = args[0]
+            else:
+                filename = os.path.expanduser(args[0])
+                what = 'all'
+        elif 2 == len(args):
+            what = args[0]
+            if args[0] not in actions:
+                self.errmsg("Action has to be 'all', 'break' or 'settings; " 
+                            "got '%s'." % args[0])
+                return False
+            filename = os.path.expanduser(args[1])
+        else: 
+            # len(args) > 2:
+            self.errmsg("Expecting 0-2 arguments, got %d." % len(args))
+            return False
+        restart_file = open(filename, 'w')
+        what_str = ''
+        if what in ('all', 'break'):
+            lines = self.output_break_commands()
+            restart_file.write("\n".join(lines) + "\n")
+            what_str = 'Breakpoints'
+        if what in ('all', 'settings'):
+            if '' == what_str:
+                what_str = 'Settings'
+            else:
+                what_str = 'Settings and breakpoints'
+            for subcommand in self.setcmds.list():
+                if subcommand in ['args', 'debug-pydb', 'history', 'logging', 
+                                  'prompt', 'trace-commands', 'warnoptions']: 
+                                  continue
+                # commands that have lots of output we can't handle right now.
+                if not self.showcmds.subcmds[subcommand]['in_list']:
+                    continue
+                val = eval('self.get_%s()' % subcommand)
+                restart_file.write('set %s %s\n' % (subcommand, val))
+        if what_str != '':
+            restart_file.close()
+            self.msg('%s saved to file %s' % (what_str, restart_file.name))
+            return False
+        return False
+
     def do_set(self, arg):
         """See help_set"""
         args = arg.split()
@@ -1593,7 +1809,7 @@ assignment to the variable __return__."""
             return
         if len(args) < 2:
             self.errmsg("Expecting at least 2 arguments, got %d." % len(args))
-            return
+            return False
         self.setcmds.do(self, args[0], args)
         return False
 
@@ -1651,7 +1867,7 @@ Send a signal to the debugged process.
         
     def do_source(self, arg):
         """source FILE
-        Read commands from a file named FILE.
+        Read debugger commands from a file named FILE.
         Note that the file '.pydbrc' is read automatically
         in this way when pydb is started.
 
