@@ -1,18 +1,18 @@
-#!/usr/local/bin/python -t
+#!/usr/bin/python -t
 """A Python debugger. Contains user-callable routines, e.g.
 exception_hook, run, runl, runv, debugger (AKA set_trace), post_mortem,
 or runevel.
 
 (See also pydb.doc for documentation.)
 
-$Id: pydb.py.in,v 1.151 2007/04/13 10:00:35 rockyb Exp $"""
+$Id: pydb.py.in,v 1.155 2008/05/21 11:02:51 rockyb Exp $"""
 
 # This part contains user entry subroutines.
 
 __title__ = 'pydb'
 
 # The name of the debugger we are currently going by.
-__version__ = '1.22'
+__version__ = '1.23'
 
 import inspect, os, re, sys, traceback
 
@@ -98,6 +98,21 @@ def process_options(p, debugger_name, pkg_version, option_list=None):
                          action="store_true", default=False,
                          help="Start off with threading debug support")
 
+    # annotate option produces annotations, used in pydb.el for a better emacs
+    # integration. Annotations are similar in purpose to those of GDB (see
+    # that manual for a description), although the syntax is different.
+    # they have the following format:
+    #
+    # ^Z^Zannotname
+    # <arbitrary text>
+    # ^Z^Z
+    #
+    # where ^Z is the ctrl-Z character, and "annotname" is the name of the
+    # annotation. A line with only two ^Z ends the annotation (no nesting
+    # allowed). See pydb.el for the usage
+    optparser.add_option("--annotate", default=0, type="int",
+                         help="Use annotations (to work inside emacs)")
+
     # Set up to stop on the first non-option because that's the name
     # of the script to be debugged on arguments following that are
     # that scripts options that should be left untouched.  We would
@@ -196,6 +211,9 @@ def process_options(p, debugger_name, pkg_version, option_list=None):
                   opts.errors
             print sys.exc_info()[0]
             sys.exit(2)
+
+    p.annotate = opts.annotate
+            
     return (opts, p)
 
 class Pdb(Gdb):
@@ -211,8 +229,45 @@ class Pdb(Gdb):
     def __init__(self, completekey='tab', stdin=None, stdout=None,
                  siglist=None):
         Gdb.__init__(self, completekey, stdin, stdout, siglist)
+        self.annotate = 0     # if > 0, print annotations (for better emacs
+                              # integration)
+        self.__show_breakpoints_postcmd = set(["break", "b", "tbreak", 
+                                               "disable", "enable", "condition",
+                                               "cl", "clear", "delete"])
+        self.__show_annotations_preloop = set(["s", "step", "c", "continue",
+                                               "n", "next", "return", "jump"])
+        self.__show_annotations_postcmd = set(["down", "frame", "up"])
+        self.__show_annotations = True
         return
 
+    def postcmd(self, stop, line):
+        if self.annotate > 0:
+            cmd = ""
+            if line: cmd = line.split()[0].strip()
+            if not cmd: cmd = self.lastcmd 
+            if cmd in self.__show_breakpoints_postcmd:
+                self.__annotation('breakpoints', self.do_info, 'breakpoints')
+            self.__show_annotations = cmd in self.__show_annotations_preloop
+            if cmd in self.__show_annotations_postcmd:
+                self.__annotation('stack', self.do_where, 0)
+                self.__annotation('locals', self.do_info, 'locals')
+        return stop
+
+    def preloop(self):
+        if self.annotate > 0:
+            if self.__show_annotations:
+                # if we are here, the stack frames have changed outside the
+                # command loop (e.g. after a "continue" command), so we show
+                # the annotations again
+                self.__annotation('breakpoints', self.do_info, 'breakpoints')
+                self.__annotation('stack', self.do_where, 0)
+                self.__annotation('locals', self.do_info, 'locals')
+
+    def __annotation(self, label, cmd, *args):
+        self.msg(chr(26) + chr(26) + label) # chr(26) is Ctrl-Z
+        cmd(*args)
+        self.msg(chr(26) + chr(26))
+
 # -- end class Pdb
 
 #########################################################
@@ -406,7 +461,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1, p=None):
         p._program_sys_argv[:0] = [p.mainpyfile]
 
     if 0 == len(p._sys_argv):
-        # Fake script invocation (restart) args since we don don't have any
+        # Fake script invocation (restart) args since we don't have any
         p._sys_argv = list(p._program_sys_argv)
         p._sys_argv[:0] = [__title__]
 
