@@ -1,4 +1,4 @@
-"""$Id: pydbbdb.py,v 1.40 2007/04/11 09:28:50 rockyb Exp $
+"""$Id: pydbbdb.py,v 1.44 2008/05/17 10:08:33 rockyb Exp $
 Routines here have to do with the subclassing of bdb.  Defines Python
 debugger Basic Debugger (Bdb) class.  This file could/should probably
 get merged into bdb.py
@@ -33,8 +33,17 @@ class Bdb(bdb.Bdb):
         # A 0 value means stop on this occurrence. A positive value means to
         # skip that many more step/next's.
         self.step_ignore      = 0
+
+        # Do we want to show/stop at def statements before they are run?
+        self.deftrace         = False
         return
 
+    def __print_call_params(self, frame):
+        "Show call paramaters and values"
+        self.setup(frame)
+        self.msg(self.format_stack_entry(self.stack[-1],
+                                         include_location=False))
+
     def __print_location_if_trace(self, frame, include_fntrace=True):
         if self.linetrace or (self.fntrace and include_fntrace):
             self.setup(frame)
@@ -71,7 +80,7 @@ class Bdb(bdb.Bdb):
 
     def is_running(self):
         if self.running: return True
-        self.msg('The program being debugged is not being run.')
+        self.errmsg('The program being debugged is not being run.')
         return False
 
     def lookupmodule(self, filename):
@@ -121,6 +130,25 @@ class Bdb(bdb.Bdb):
             self.msg('\tbreakpoint already hit %d time%s' %
                      (bp.hits, ss), out)
 
+    def output_break_commands(self):
+        "Output a list of 'break' commands"
+        # FIXME: for now we ae going to assume no breakpoints set
+        # previously
+        bp_no = 0 
+        out = []
+        for bp in bdb.Breakpoint.bpbynumber:
+            if bp:
+                bp_no += 1
+                if bp.cond: 
+                    condition = bp.cond
+                else:
+                    condition = ''
+                out.append("break %s:%s%s" % 
+                           (self.filename(bp.file), bp.line, condition))
+                if not bp.enabled:
+                    out.append("disable %s" % bp_no)
+        return out
+
     def break_here(self, frame):
         """This routine is almost copy of bdb.py's routine. Alas what pdb
         calls clear gdb calls delete and gdb's clear command is different.
@@ -235,7 +263,8 @@ class Bdb(bdb.Bdb):
             return(os.path.basename(filename))
         return filename
 
-    def format_stack_entry(self, frame_lineno, lprefix=': '):
+    def format_stack_entry(self, frame_lineno, lprefix=': ',
+                           include_location=True):
         """Format and return a stack entry gdb-style.
         Note: lprefix is not used. It is kept for compatibility.
         """
@@ -265,20 +294,21 @@ class Bdb(bdb.Bdb):
             s += repr_mod.repr(rv)
 
         add_quotes_around_file = True
-        if s == '?()':
-            if is_exec_stmt(frame):
-                s = 'in exec'
-                exec_str = get_exec_string(frame.f_back)
-                if exec_str != None:
-                    filename = exec_str
-                    add_quotes_around_file = False
+        if include_location:
+            if s == '?()':
+                if is_exec_stmt(frame):
+                    s = 'in exec'
+                    exec_str = get_exec_string(frame.f_back)
+                    if exec_str != None:
+                        filename = exec_str
+                        add_quotes_around_file = False
+                else:
+                    s = 'in file'
             else:
-                s = 'in file'
-        else:
-            s += ' called from file'
+                s += ' called from file'
 
-        if add_quotes_around_file: filename = "'%s'" % filename
-        s += " %s at line %r" % (filename, lineno)
+            if add_quotes_around_file: filename = "'%s'" % filename
+            s += " %s at line %r" % (filename, lineno)
         return s
 
     # The following two methods can be called by clients to use
@@ -333,7 +363,12 @@ class Bdb(bdb.Bdb):
             return
         if self.stop_here(frame):
             frame_count = count_frames(frame, Bdb.extra_call_frames)
-            self.msg('--%sCall level %d' % ('-' * (2*frame_count), frame_count))
+            self.msg_nocr('--%sCall level %d' % 
+                          ('-' * (2*frame_count), frame_count))
+            if frame_count >= 0:
+                self.__print_call_params(frame)
+            else:
+                self.msg("")
             if self.linetrace or self.fntrace:
                 self.__print_location_if_trace(frame)
                 if not self.break_here(frame): return
@@ -385,7 +420,7 @@ class Bdb(bdb.Bdb):
                 self.__print_location_if_trace(frame, False)
                 return
             if not self.break_here(frame):
-                if is_def_stmt(line, frame):
+                if is_def_stmt(line, frame) and not self.deftrace:
                     self.__print_location_if_trace(frame, False)
                     return
                 elif self.fntrace:
