@@ -6,7 +6,7 @@ not always) they are not specific to pydb. They are sort of more
 oriented towards any gdb-like debugger. Also routines that need to be
 changed from cmd are here.
 
-$Id: pydbcmd.py,v 1.42 2007/02/20 18:21:52 rockyb Exp $"""
+$Id: pydbcmd.py,v 1.47 2008/04/16 01:20:47 rockyb Exp $"""
 
 import cmd, linecache, sys, types
 from fns import *
@@ -72,6 +72,7 @@ class Cmd(cmd.Cmd):
         statement = 'execfile( "%s")' % filename
         self.running = True
         self.run(statement, globals=globals_, locals=locals_)
+        return
 
     def default(self, line):
         """Method called on an input line when the command prefix is
@@ -80,6 +81,9 @@ class Cmd(cmd.Cmd):
 
         if line[:1] == '#': return
         if line[:1] == '$': line = line[1:]
+        if not self.autoeval: 
+            self.errmsg("""Undefined command: "%s".  Try "help".""" % line)
+            return
         if self.curframe:
             local_vars = self.curframe.f_locals
             global_vars = self.curframe.f_globals
@@ -184,9 +188,12 @@ See also 'examine' an 'whatis'.
                     else:
                         cmds_undoc.append(cmd)
             self.msg("%s\n" % str(self.doc_leader))
-            self.print_topics(self.doc_header,   cmds_doc,   15,80)
-            self.print_topics(self.misc_header,  help_dict.keys(),15,80)
-            self.print_topics(self.undoc_header, cmds_undoc, 15,80)
+            self.print_topics(self.doc_header,   cmds_doc,   15,
+                              self.width)
+            self.print_topics(self.misc_header,  help_dict.keys(),15,
+                              self.width)
+            self.print_topics(self.undoc_header, cmds_undoc, 15,
+                              self.width)
 
     do_h = do_help
 
@@ -223,6 +230,7 @@ See also 'examine' an 'whatis'.
         than get_int(). We eval arg return it as an integer value or
         None if there was an error in parsing this.
         """
+        ret_value = None
         if arg:
             try:
                 # eval() is used so we will allow arithmetic expressions,
@@ -262,8 +270,8 @@ See also 'examine' an 'whatis'.
         return default
 
     def get_onoff(self, arg, default=None, print_error=True):
-        """Return True if arg is 'on' or 1 and False arg is an 'off' or 0
-        Any other value is an error"""
+        """Return True if arg is 'on' or 1 and False arg is 'off' or 0.
+        Any other value is raises ValueError."""
         if not arg:
             if default is None:
                 if print_error:
