@@ -1,7 +1,7 @@
 """set subcommands, except those that need some sort of text substitution.
 (Those are in gdb.py.in.)
 """
-__revision__ = "$Id: set.py,v 1.17 2007/04/07 10:53:29 rockyb Exp $"
+__revision__ = "$Id: set.py,v 1.23 2008/05/17 10:08:33 rockyb Exp $"
 # -*- coding: utf-8 -*-
 #   Copyright (C) 2006, 2007 Rocky Bernstein
 #
@@ -20,7 +20,7 @@ __revision__ = "$Id: set.py,v 1.17 2007/04/07 10:53:29 rockyb Exp $"
 #    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #    02110-1301 USA.
 
-import inspect, re, sighandler, sys
+import inspect, os, re, sighandler, sys
 
 class SubcmdSet:
 
@@ -41,6 +41,17 @@ class SubcmdSet:
     ######## Note: the docstrings of methods here get used in
     ######## help output.
 
+    def set_annotate(self, args):
+        """Set annotation level
+0 == normal;     1 == fullname (for use when running under emacs)
+2 == output annotated suitably for use by programs that control GDB.
+"""
+        try:
+            self.annotate = self.get_int(args[1])
+        except ValueError:
+            pass
+        return
+
     def set_args(self, args):
         """Set argument list to give program being debugged when it is started.
 Follow this command with any number of args, to be passed to the program."""
@@ -52,6 +63,14 @@ Follow this command with any number of args, to be passed to the program."""
             self._program_sys_argv[:0] = argv_start
         return
 
+    def set_autoeval(self, args):
+        """Evaluate every unrecognized command."""
+        try:
+            self.autoeval = self.get_onoff(args[1])
+        except ValueError:
+            pass
+        return
+
     def set_basename(self, args):
         """Set short filenames (the basename) in debug output"""
         try:
@@ -110,6 +129,14 @@ object variables.
         self.msg('debug-signal set to: %s' % self.debug_signal)
         return False
 
+    def set_deftrace(self, args):
+        """Set to def's (method creation) before they are run"""
+        try:
+            self.deftrace = self.get_onoff(args[1])
+        except ValueError:
+            pass
+        return
+
     def set_flush(self, args):
         """Set whether we flush output after each write."""
         try:
@@ -127,7 +154,12 @@ object variables.
         return
 
     def set_history(self, args):
-        """Generic command for setting command history parameters."""
+        """Generic command for setting command history parameters.
+
+set history filename - set location to save history
+set history save [on|off] - specify whether or not ot save history
+set history size n - set number of commands to save in history
+        """
         if args[1] == 'filename':
             if len(args) < 3:
                 self.errmsg("Argument required (filename to set it to).")
@@ -290,3 +322,14 @@ Examples:
         sys.warnoptions = args[1:]
         self.show_warnoptions(args)
         return
+
+    def set_width(self, args):
+        """Set number of characters the debugger thinks are in a line.
+We also change OS environment variable COLUMNS."""
+        try:
+            self.width = self.get_int(args[1])
+            os.environ['COLUMNS'] = args[1]
+        except ValueError:
+            pass
+        return
+
