@@ -1,9 +1,9 @@
-#!/usr/local/bin/python -t
+#!/usr/bin/python -t
 # $Id: except.py.in,v 1.4 2007/01/25 19:04:45 rockyb Exp $
 """Test to see if handles an exception when an error is raised."""
 import os, sys
 
-top_builddir = "../"
+top_builddir = ".."
 if top_builddir[-1] != os.path.sep:
     top_builddir += os.path.sep
 sys.path.insert(0, os.path.join(top_builddir, 'pydb'))
